@@ -124,5 +124,5 @@ method! reload_test tst arg =
 
 end
 
-let fundecl f =
-  (new reload)#fundecl f
+let fundecl f num_stack_slots =
+  (new reload)#fundecl f num_stack_slots
