@@ -219,10 +219,6 @@ let string_of_rounding = function
   | RoundTruncate -> "roundsd.trunc"
   | RoundNearest -> "roundsd.near"
 
-(* These hooks can be used to insert optimization passes on
-   the assembly code. *)
-let assembler_passes = ref ([] : (asm_program -> asm_program) list)
-
 let internal_assembler = ref None
 let register_internal_assembler f = internal_assembler := Some f
 
@@ -268,9 +264,6 @@ let reset_asm_code () = asm_code := []
 
 let generate_code asm =
   let instrs = List.rev !asm_code in
-  let instrs =
-    List.fold_left (fun instrs pass -> pass instrs) instrs !assembler_passes
-  in
   begin match asm with
   | Some f -> f instrs
   | None -> ()
