@@ -24,9 +24,9 @@ open Debugger_config
 open Types
 open Primitives
 open Unix_tools
-open Parser
+open Debugger_parser
 open Parser_aux
-open Lexer
+open Debugger_lexer
 open Input_handling
 open Question
 open Debugcom
@@ -45,6 +45,8 @@ open Checkpoints
 open Frames
 open Printval
 
+module Lexer = Debugger_lexer
+
 (** Instructions, variables and infos lists. **)
 type dbg_instruction =
   { instr_name: string;                 (* Name of command *)
@@ -126,14 +128,15 @@ let add_breakpoint_at_pc pc =
     new_breakpoint (any_event_at_pc pc)
   with
   | Not_found ->
-    eprintf "Can\'t add breakpoint at pc %i: no event there.@." pc;
+    eprintf "Can\'t add breakpoint at pc %i:%i: no event there.@."
+            pc.frag pc.pos;
     raise Toplevel
 
 let add_breakpoint_after_pc pc =
   let rec try_add n =
     if n < 3 then begin
       try
-        new_breakpoint (any_event_at_pc (pc + n * 4))
+        new_breakpoint (any_event_at_pc {pc with pos = pc.pos + n * 4})
       with
       | Not_found ->
         try_add (n+1)
@@ -156,11 +159,8 @@ let convert_module mdle =
                               then Filename.chop_suffix m ".ml"
                               else m)
   | None ->
-      try
-        (get_current_event ()).ev_module
-      with
-      | Not_found ->
-          error "Not in a module."
+      try (get_current_event ()).ev_ev.ev_module
+      with Not_found -> error "Not in a module."
 
 (** Toplevel. **)
 let current_line = ref ""
@@ -210,7 +210,7 @@ let line_loop ppf line_buffer =
       done
     with
     | Exit ->
-        stop_user_input ()
+        ()
 (*    | Sys_error s ->
         error ("System error: " ^ s) *)
 
@@ -303,7 +303,7 @@ let instr_run ppf lexbuf =
   ensure_loaded ();
   reset_named_values ();
   run ();
-  show_current_event ppf;;
+  show_current_event ppf
 
 let instr_reverse ppf lexbuf =
   eol lexbuf;
@@ -502,7 +502,7 @@ let env_of_event =
   function
     None    -> Env.empty
   | Some ev ->
-      Envaux.env_from_summary ev.Instruct.ev_typenv ev.Instruct.ev_typsubst
+      Envaux.env_from_summary ev.ev_ev.ev_typenv ev.ev_ev.ev_typsubst
 
 let print_command depth ppf lexbuf =
   let exprs = expression_list_eol Lexer.lexeme lexbuf in
@@ -567,20 +567,17 @@ let instr_source ppf lexbuf =
       | Not_found -> error "Source file not found."
       | (Unix_error _) as x  -> Unix_tools.report_error x; raise Toplevel
     in
-      try
-        interactif := false;
-        user_channel := io_chan;
-        line_loop ppf (Lexing.from_function read_user_input);
+      interactif := false;
+      user_channel := io_chan;
+      let loop () =
+        line_loop ppf (Lexing.from_function read_user_input)
+      and finally () =
+        stop_user_input ();
         close_io io_chan;
         interactif := old_state;
         user_channel := old_channel
-      with
-      | x ->
-          stop_user_input ();
-          close_io io_chan;
-          interactif := old_state;
-          user_channel := old_channel;
-          raise x
+      in
+      Fun.protect ~finally loop
 
 let instr_set =
   find_variable
@@ -613,8 +610,8 @@ let instr_break ppf lexbuf =
              new_breakpoint ev
          | None ->
              error "Can\'t add breakpoint at this point.")
-    | BA_pc pc ->                               (* break PC *)
-        add_breakpoint_at_pc pc
+    | BA_pc {frag; pos} ->                      (* break PC *)
+        add_breakpoint_at_pc {frag; pos}
     | BA_function expr ->                       (* break FUNCTION *)
         let env =
           try
@@ -644,7 +641,7 @@ let instr_break ppf lexbuf =
             let ev =  event_at_pos module_name 0 in
             let ev_pos =
               {Lexing.dummy_pos with
-               pos_fname = (Events.get_pos ev).pos_fname} in
+               pos_fname = (Events.get_pos ev.ev_ev).pos_fname} in
              let buffer =
                try get_buffer ev_pos module_name with
                | Not_found ->
@@ -703,7 +700,7 @@ let instr_backtrace ppf lexbuf =
     | Some x -> x in
   ensure_loaded ();
   match current_report() with
-  | None | Some {rep_type = Exited | Uncaught_exc} -> ()
+  | None | Some {rep_type = Exited | Uncaught_exc | Code_loaded _} -> ()
   | Some _ ->
       let frame_counter = ref 0 in
       let print_frame first_frame last_frame = function
@@ -936,8 +933,8 @@ let info_checkpoints ppf lexbuf =
           !checkpoints))
 
 let info_one_breakpoint ppf (num, ev) =
-  fprintf ppf "%3d %10d  %s@." num ev.ev_pos (Pos.get_desc ev);
-;;
+  fprintf ppf "%3d %d:%10d  %s@." num ev.ev_frag ev.ev_ev.ev_pos
+          (Pos.get_desc ev)
 
 let info_breakpoints ppf lexbuf =
   eol lexbuf;
@@ -946,7 +943,7 @@ let info_breakpoints ppf lexbuf =
     fprintf ppf "Num    Address  Where@.";
     List.iter (info_one_breakpoint ppf) (List.rev !breakpoints);
   end
-;;
+
 
 let info_events _ppf lexbuf =
   ensure_loaded ();
@@ -955,6 +952,7 @@ let info_events _ppf lexbuf =
   in
     print_endline ("Module: " ^ mdle);
     print_endline "   Address  Characters        Kind      Repr.";
+    let frag, events = events_in_module mdle in
     List.iter
       (function ev ->
         let start_char, end_char =
@@ -966,7 +964,8 @@ let info_events _ppf lexbuf =
             ev.ev_loc.Location.loc_start.Lexing.pos_cnum,
             ev.ev_loc.Location.loc_end.Lexing.pos_cnum in
         Printf.printf
-           "%10d %6d-%-6d  %10s %10s\n"
+           "%d:%10d %6d-%-6d  %10s %10s\n"
+           frag
            ev.ev_pos
            start_char
            end_char
@@ -983,7 +982,7 @@ let info_events _ppf lexbuf =
               Event_none        -> ""
             | Event_parent _    -> "(repr)"
             | Event_child repr  -> Int.to_string !repr))
-      (events_in_module mdle)
+      events
 
 (** User-defined printers **)
 
@@ -1093,10 +1092,14 @@ Argument N means do this N times (or till program stops for another reason)." };
      (* Breakpoints *)
      { instr_name = "break"; instr_prio = false;
        instr_action = instr_break; instr_repeat = false; instr_help =
-"Set breakpoint at specified line or function.\
-\nSyntax: break function-name\
+"Set breakpoint.\
+\nSyntax: break\
+\n        break function-name\
 \n        break @ [module] linenum\
-\n        break @ [module] # characternum" };
+\n        break @ [module] linenum columnnum\
+\n        break @ [module] # characternum\
+\n        break frag:pc\
+\n        break pc" };
      { instr_name = "delete"; instr_prio = false;
        instr_action = instr_delete; instr_repeat = false; instr_help =
 "delete some breakpoints.\n\
@@ -1214,7 +1217,11 @@ It can be either:\n\
 "process to follow after forking.\n\
 It can be either :\n\
   child: the newly created process.\n\
-  parent: the process that called fork.\n" }];
+  parent: the process that called fork.\n" };
+     { var_name = "break_on_load";
+       var_action = boolean_variable false break_on_load;
+       var_help =
+"whether to stop after loading new code (e.g. with Dynlink)." }];
 
   info_list :=
     (* info name, function, help *)
