@@ -29,14 +29,13 @@ val first_ppx : string list ref
 val first_include_dirs : string list ref
 val last_include_dirs : string list ref
 
-(* function to call on plugin=XXX *)
-val load_plugin : (string -> unit) ref
-
 (* return the list of objfiles, after OCAMLPARAM and List.rev *)
 val get_objfiles : with_ocamlparam:bool -> string list
 val last_objfiles : string list ref
 val first_objfiles : string list ref
 
+val stop_early : bool ref
+
 type filename = string
 
 type readenv_position =
