@@ -62,6 +62,12 @@ let dumpenv = make
   (fun log env ->
     Environments.dump log env; (Result.pass, env))
 
+let hasinstrumentedruntime = make
+  "hasinstrumentedruntime"
+  (Actions_helpers.pass_or_skip (Ocamltest_config.has_instrumented_runtime)
+    "instrumented runtime available"
+    "instrumented runtime not available")
+
 let hasunix = make
   "hasunix"
   (Actions_helpers.pass_or_skip (Ocamltest_config.libunix <> None)
@@ -145,6 +151,42 @@ let arch64 = make
     "64-bit architecture"
     "non-64-bit architecture")
 
+let arch_arm = make
+  "arch_arm"
+  (Actions_helpers.pass_or_skip (String.equal Ocamltest_config.arch "arm")
+     "Target is ARM architecture"
+     "Target is not ARM architecture")
+
+let arch_arm64 = make
+  "arch_arm64"
+  (Actions_helpers.pass_or_skip (String.equal Ocamltest_config.arch "arm64")
+     "Target is ARM64 architecture"
+     "Target is not ARM64 architecture")
+
+ let arch_amd64 = make
+  "arch_amd64"
+  (Actions_helpers.pass_or_skip (String.equal Ocamltest_config.arch "amd64")
+     "Target is AMD64 architecture"
+     "Target is not AMD64 architecture")
+
+ let arch_i386 = make
+  "arch_i386"
+  (Actions_helpers.pass_or_skip (String.equal Ocamltest_config.arch "i386")
+     "Target is i386 architecture"
+     "Target is not i386 architecture")
+
+let arch_power = make
+  "arch_power"
+  (Actions_helpers.pass_or_skip (String.equal Ocamltest_config.arch "power")
+    "Target is POWER architecture"
+    "Target is not POWER architecture")
+
+let function_sections = make
+  "function_sections"
+  (Actions_helpers.pass_or_skip (Ocamltest_config.function_sections)
+     "Target supports function sections"
+     "Target does not support function sections")
+
 let has_symlink = make
   "has_symlink"
   (Actions_helpers.pass_or_skip (Sys.has_symlink () )
@@ -191,6 +233,7 @@ let _ =
     fail;
     cd;
     dumpenv;
+    hasinstrumentedruntime;
     hasunix;
     hassysthreads;
     hasstr;
@@ -208,4 +251,10 @@ let _ =
     run;
     script;
     check_program_output;
+    arch_arm;
+    arch_arm64;
+    arch_amd64;
+    arch_i386;
+    arch_power;
+    function_sections;
   ]
