@@ -1071,7 +1071,7 @@ type socket_type = Unix.socket_type =
 
 type sockaddr = Unix.sockaddr =
     ADDR_UNIX of string
-  | ADDR_INET of inet_addr * int
+  | ADDR_INET of inet_addr * int (**)
 (** The type of socket addresses. [ADDR_UNIX name] is a socket
    address in the Unix domain; [name] is a file name in the file
    system. [ADDR_INET(addr,port)] is a socket address in the Internet
@@ -1131,7 +1131,7 @@ val getpeername : file_descr -> sockaddr
 type msg_flag = Unix.msg_flag =
     MSG_OOB
   | MSG_DONTROUTE
-  | MSG_PEEK
+  | MSG_PEEK (**)
 (** The flags for {!UnixLabels.recv},  {!UnixLabels.recvfrom},
    {!UnixLabels.send} and {!UnixLabels.sendto}. *)
 
