@@ -26,31 +26,12 @@
 #include "caml/osdeps.h"
 #include "unixsupport.h"
 
-/* Temporary compatibility stuff so that this file can also be compiled
-   from otherlibs/bigarray/ and included in the bigarray library. */
-
-#ifdef IN_OCAML_BIGARRAY
-#define MAP_FILE_FUNCTION caml_ba_map_file
-#define MAP_FILE_FUNCTION_BYTECODE caml_ba_map_file_bytecode
-#define UNMAP_FILE_FUNCTION caml_ba_unmap_file
-#define ALLOC_FUNCTION caml_ba_mapped_alloc
-#define CAML_MAP_FILE "Bigarray.map_file"
-static void caml_ba_sys_error(void);
-#define MAP_FILE_ERROR() caml_ba_sys_error()
-#else
-#define MAP_FILE_FUNCTION caml_unix_map_file
-#define MAP_FILE_FUNCTION_BYTECODE caml_unix_map_file_bytecode
-#define UNMAP_FILE_FUNCTION caml_unix_unmap_file
-#define ALLOC_FUNCTION caml_unix_mapped_alloc
-#define MAP_FILE_FUNCTION caml_unix_map_file
-#define CAML_MAP_FILE "Unix.map_file"
-#define MAP_FILE_ERROR() \
-  do { win32_maperr(GetLastError()); uerror("map_file", Nothing); } while(0)
-#endif
+#define uerror(func, arg) \
+  do { win32_maperr(GetLastError()); uerror(func, arg); } while(0)
 
 /* Defined in [mmap_ba.c] */
 CAMLextern value
-ALLOC_FUNCTION(int flags, int num_dims, void * data, intnat * dim);
+caml_unix_mapped_alloc(int flags, int num_dims, void * data, intnat * dim);
 
 #ifndef INVALID_SET_FILE_POINTER
 #define INVALID_SET_FILE_POINTER (-1)
@@ -67,8 +48,8 @@ static __int64 caml_set_file_pointer(HANDLE h, __int64 dist, DWORD mode)
   return i.QuadPart;
 }
 
-CAMLprim value MAP_FILE_FUNCTION(value vfd, value vkind, value vlayout,
-                                 value vshared, value vdim, value vstart)
+CAMLprim value caml_unix_map_file(value vfd, value vkind, value vlayout,
+                                  value vshared, value vdim, value vstart)
 {
   HANDLE fd, fmap;
   int flags, major_dim, mode, perm;
@@ -89,18 +70,18 @@ CAMLprim value MAP_FILE_FUNCTION(value vfd, value vkind, value vlayout,
   /* Extract dimensions from OCaml array */
   num_dims = Wosize_val(vdim);
   if (num_dims < 1 || num_dims > CAML_BA_MAX_NUM_DIMS)
-    caml_invalid_argument(CAML_MAP_FILE ": bad number of dimensions");
+    caml_invalid_argument("Unix.map_file: bad number of dimensions");
   for (i = 0; i < num_dims; i++) {
     dim[i] = Long_val(Field(vdim, i));
     if (dim[i] == -1 && i == major_dim) continue;
     if (dim[i] < 0)
-      caml_invalid_argument(CAML_MAP_FILE ": negative dimension");
+      caml_invalid_argument("Unix.map_file: negative dimension");
   }
   /* Determine file size */
   currpos = caml_set_file_pointer(fd, 0, FILE_CURRENT);
-  if (currpos == -1) MAP_FILE_ERROR();
+  if (currpos == -1) uerror("map_file", Nothing);
   file_size = caml_set_file_pointer(fd, 0, FILE_END);
-  if (file_size == -1) MAP_FILE_ERROR();
+  if (file_size == -1) uerror("map_file", Nothing);
   /* Determine array size in bytes (or size of array without the major
      dimension if that dimension wasn't specified) */
   array_size = caml_ba_element_size[flags & CAML_BA_KIND_MASK];
@@ -110,12 +91,12 @@ CAMLprim value MAP_FILE_FUNCTION(value vfd, value vkind, value vlayout,
   if (dim[major_dim] == -1) {
     /* Determine first/last dimension from file size */
     if (file_size < startpos)
-      caml_failwith(CAML_MAP_FILE ": file position exceeds file size");
+      caml_failwith("Unix.map_file: file position exceeds file size");
     data_size = file_size - startpos;
     dim[major_dim] = (uintnat) (data_size / array_size);
     array_size = dim[major_dim] * array_size;
     if (array_size != data_size)
-      caml_failwith(CAML_MAP_FILE ": file size doesn't match array dimensions");
+      caml_failwith("Unix.map_file: file size doesn't match array dimensions");
   }
   /* Restore original file position */
   caml_set_file_pointer(fd, currpos, FILE_BEGIN);
@@ -129,7 +110,7 @@ CAMLprim value MAP_FILE_FUNCTION(value vfd, value vkind, value vlayout,
   }
   li.QuadPart = startpos + array_size;
   fmap = CreateFileMapping(fd, NULL, perm, li.HighPart, li.LowPart, NULL);
-  if (fmap == NULL) MAP_FILE_ERROR();
+  if (fmap == NULL) uerror("map_file", Nothing);
   /* Determine offset so that the mapping starts at the given file pos */
   GetSystemInfo(&sysinfo);
   delta = (uintnat) (startpos % sysinfo.dwAllocationGranularity);
@@ -137,21 +118,21 @@ CAMLprim value MAP_FILE_FUNCTION(value vfd, value vkind, value vlayout,
   li.QuadPart = startpos - delta;
   addr =
     MapViewOfFile(fmap, mode, li.HighPart, li.LowPart, array_size + delta);
-  if (addr == NULL) MAP_FILE_ERROR();
+  if (addr == NULL) uerror("map_file", Nothing);
   addr = (void *) ((uintnat) addr + delta);
   /* Close the file mapping */
   CloseHandle(fmap);
   /* Build and return the OCaml bigarray */
-  return ALLOC_FUNCTION(flags, num_dims, addr, dim);
+  return caml_unix_mapped_alloc(flags, num_dims, addr, dim);
 }
 
-CAMLprim value MAP_FILE_FUNCTION_BYTECODE(value * argv, int argn)
+CAMLprim value caml_unix_map_file_bytecode(value * argv, int argn)
 {
-  return MAP_FILE_FUNCTION(argv[0], argv[1], argv[2],
-                           argv[3], argv[4], argv[5]);
+  return caml_unix_map_file(argv[0], argv[1], argv[2],
+                            argv[3], argv[4], argv[5]);
 }
 
-void UNMAP_FILE_FUNCTION(void * addr, uintnat len)
+void caml_ba_unmap_file(void * addr, uintnat len)
 {
   SYSTEM_INFO sysinfo;
   uintnat delta;
