@@ -265,7 +265,7 @@ int error_table[] = {
   EHOSTUNREACH, ELOOP, EOVERFLOW /*, EUNKNOWNERR */
 };
 
-static value * unix_error_exn = NULL;
+static const value * unix_error_exn = NULL;
 
 value unix_error_of_code (int errcode)
 {
