@@ -281,7 +281,7 @@ CAMLprim value caml_hexstring_of_float(value arg, value vprec, value vstyle)
   return res;
 }
 
-static int caml_float_of_hex(const char * s, double * res)
+static int caml_float_of_hex(const char * s, const char * end, double * res)
 {
   int64_t m = 0;                /* the mantissa - top 60 bits at most */
   int n_bits = 0;               /* total number of bits read */
@@ -293,11 +293,9 @@ static int caml_float_of_hex(const char * s, double * res)
   char * p;                     /* for converting the exponent */
   double f;
 
-  while (*s != 0) {
+  while (s < end) {
     char c = *s++;
     switch (c) {
-    case '_':
-      break;
     case '.':
       if (dec_point >= 0) return -1; /* multiple decimal points */
       dec_point = n_bits;
@@ -306,7 +304,7 @@ static int caml_float_of_hex(const char * s, double * res)
       long e;
       if (*s == 0) return -1;   /* nothing after exponent mark */
       e = strtol(s, &p, 10);
-      if (*p != 0) return -1;   /* ill-formed exponent */
+      if (p != end) return -1;  /* ill-formed exponent */
       /* Handle exponents larger than int by returning 0/infinity directly.
          Mind that INT_MIN/INT_MAX are included in the test so as to capture
          the overflow case of strtol on Win64 -- long and int have the same
@@ -381,17 +379,7 @@ CAMLprim value caml_float_of_string(value vs)
   int sign;
   double d;
 
-  /* Check for hexadecimal FP constant */
-  src = String_val(vs);
-  sign = 1;
-  if (*src == '-') { sign = -1; src++; }
-  else if (*src == '+') { src++; };
-  if (src[0] == '0' && (src[1] == 'x' || src[1] == 'X')) {
-    if (caml_float_of_hex(src + 2, &d) == -1)
-      caml_failwith("float_of_string");
-    return caml_copy_double(sign < 0 ? -d : d);
-  }
-  /* Remove '_' characters before calling strtod () */
+  /* Remove '_' characters before conversion */
   len = caml_string_length(vs);
   buf = len < sizeof(parse_buffer) ? parse_buffer : caml_stat_alloc(len + 1);
   src = String_val(vs);
@@ -402,15 +390,26 @@ CAMLprim value caml_float_of_string(value vs)
   }
   *dst = 0;
   if (dst == buf) goto error;
+  /* Check for hexadecimal FP constant */
+  src = buf;
+  sign = 1;
+  if (*src == '-') { sign = -1; src++; }
+  else if (*src == '+') { src++; };
+  if (src[0] == '0' && (src[1] == 'x' || src[1] == 'X')) {
+    /* Convert using our hexadecimal FP parser */
+    if (caml_float_of_hex(src + 2, dst, &d) == -1) goto error;
+    if (sign < 0) d = -d;
+  } else {
+    /* Convert using strtod */
 #if defined(HAS_STRTOD_L) && defined(HAS_LOCALE)
-  d = strtod_l((const char *) buf, &end, caml_locale);
+    d = strtod_l((const char *) buf, &end, caml_locale);
 #else
-  USE_LOCALE;
-  /* Convert using strtod */
-  d = strtod((const char *) buf, &end);
-  RESTORE_LOCALE;
+    USE_LOCALE;
+    d = strtod((const char *) buf, &end);
+    RESTORE_LOCALE;
 #endif /* HAS_STRTOD_L */
-  if (end != dst) goto error;
+    if (end != dst) goto error;
+  }
   if (buf != parse_buffer) caml_stat_free(buf);
   return caml_copy_double(d);
  error:
@@ -966,33 +965,6 @@ CAMLprim value caml_signbit_float(value f)
   return caml_signbit(Double_val(f));
 }
 
-#ifdef LACKS_SANE_NAN
-
-CAMLprim value caml_neq_float(value vf, value vg)
-{
-  double f = Double_val(vf);
-  double g = Double_val(vg);
-  return Val_bool(isnan(f) || isnan(g) || f != g);
-}
-
-#define DEFINE_NAN_CMP(op) (value vf, value vg) \
-{ \
-  double f = Double_val(vf); \
-  double g = Double_val(vg); \
-  return Val_bool(!isnan(f) && !isnan(g) && f op g); \
-}
-
-intnat caml_float_compare_unboxed(double f, double g)
-{
-  /* Insane => nan == everything && nan < everything && nan > everything */
-  if (isnan(f) && isnan(g)) return 0;
-  if (!isnan(g) && f < g) return -1;
-  if (f != g) return 1;
-  return 0;
-}
-
-#else
-
 CAMLprim value caml_neq_float(value f, value g)
 {
   return Val_bool(Double_val(f) != Double_val(g));
@@ -1008,12 +980,15 @@ intnat caml_float_compare_unboxed(double f, double g)
   /* If one or both of f and g is NaN, order according to the convention
      NaN = NaN and NaN < x for all other floats x. */
   /* This branchless implementation is from GPR#164.
-     Note that [f == f] if and only if f is not NaN. */
-  return (f > g) - (f < g) + (f == f) - (g == g);
+     Note that [f == f] if and only if f is not NaN.
+     We expand each subresult of the expression to
+     avoid sign-extension on 64bit. GPR#2250.
+     See also translation of Pcompare_floats in asmcomp/cmmgen.ml  */
+  intnat res =
+    (intnat)(f > g) - (intnat)(f < g) + (intnat)(f == f) - (intnat)(g == g);
+  return res;
 }
 
-#endif
-
 CAMLprim value caml_eq_float DEFINE_NAN_CMP(==)
 CAMLprim value caml_le_float DEFINE_NAN_CMP(<=)
 CAMLprim value caml_lt_float DEFINE_NAN_CMP(<)
