@@ -30,6 +30,7 @@
 #include "caml/misc.h"
 #include "caml/mlvalues.h"
 #include "caml/signals.h"
+#include "caml/eventlog.h"
 #ifdef NATIVE_CODE
 #include "caml/stack.h"
 #else
@@ -41,17 +42,6 @@
 extern uintnat caml_max_stack_size;    /* defined in stacks.c */
 #endif
 
-double caml_stat_minor_words = 0.0,
-       caml_stat_promoted_words = 0.0,
-       caml_stat_major_words = 0.0;
-
-intnat caml_stat_minor_collections = 0,
-       caml_stat_major_collections = 0,
-       caml_stat_heap_wsz = 0,
-       caml_stat_top_heap_wsz = 0,
-       caml_stat_compactions = 0,
-       caml_stat_heap_chunks = 0;
-
 extern uintnat caml_major_heap_increment; /* percent or words; see major_gc.c */
 extern uintnat caml_percent_free;         /*        see major_gc.c */
 extern uintnat caml_percent_max;          /*        see compact.c */
@@ -223,24 +213,27 @@ static value heap_stats (int returnstats)
 
 #ifdef DEBUG
   caml_final_invariant_check();
+  caml_fl_check ();
 #endif
 
-  CAMLassert (heap_chunks == caml_stat_heap_chunks);
-  CAMLassert (live_words + free_words + fragments == caml_stat_heap_wsz);
+  CAMLassert (heap_chunks == Caml_state->stat_heap_chunks);
+  CAMLassert (live_words + free_words + fragments == Caml_state->stat_heap_wsz);
 
   if (returnstats){
     CAMLlocal1 (res);
 
     /* get a copy of these before allocating anything... */
-    double minwords = caml_stat_minor_words
-                      + (double) (caml_young_alloc_end - caml_young_ptr);
-    double prowords = caml_stat_promoted_words;
-    double majwords = caml_stat_major_words + (double) caml_allocated_words;
-    intnat mincoll = caml_stat_minor_collections;
-    intnat majcoll = caml_stat_major_collections;
-    intnat heap_words = caml_stat_heap_wsz;
-    intnat cpct = caml_stat_compactions;
-    intnat top_heap_words = caml_stat_top_heap_wsz;
+    double minwords =
+      Caml_state->stat_minor_words
+      + (double) (Caml_state->young_alloc_end - Caml_state->young_ptr);
+    double prowords = Caml_state->stat_promoted_words;
+    double majwords =
+      Caml_state->stat_major_words + (double) caml_allocated_words;
+    intnat mincoll = Caml_state->stat_minor_collections;
+    intnat majcoll = Caml_state->stat_major_collections;
+    intnat heap_words = Caml_state->stat_heap_wsz;
+    intnat cpct = Caml_state->stat_compactions;
+    intnat top_heap_words = Caml_state->stat_top_heap_wsz;
 
     res = caml_alloc_tuple (16);
     Store_field (res, 0, caml_copy_double (minwords));
@@ -275,10 +268,10 @@ void caml_heap_check (void)
 CAMLprim value caml_gc_stat(value v)
 {
   value result;
-  CAML_INSTR_SETUP (tmr, "");
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_STAT);
   CAMLassert (v == Val_unit);
   result = heap_stats (1);
-  CAML_INSTR_TIME (tmr, "explicit/gc_stat");
+  CAML_EV_END(EV_EXPLICIT_GC_STAT);
   return result;
 }
 
@@ -288,16 +281,18 @@ CAMLprim value caml_gc_quick_stat(value v)
   CAMLlocal1 (res);
 
   /* get a copy of these before allocating anything... */
-  double minwords = caml_stat_minor_words
-                    + (double) (caml_young_alloc_end - caml_young_ptr);
-  double prowords = caml_stat_promoted_words;
-  double majwords = caml_stat_major_words + (double) caml_allocated_words;
-  intnat mincoll = caml_stat_minor_collections;
-  intnat majcoll = caml_stat_major_collections;
-  intnat heap_words = caml_stat_heap_wsz;
-  intnat top_heap_words = caml_stat_top_heap_wsz;
-  intnat cpct = caml_stat_compactions;
-  intnat heap_chunks = caml_stat_heap_chunks;
+  double minwords =
+    Caml_state->stat_minor_words
+    + (double) (Caml_state->young_alloc_end - Caml_state->young_ptr);
+  double prowords = Caml_state->stat_promoted_words;
+  double majwords =
+    Caml_state->stat_major_words + (double) caml_allocated_words;
+  intnat mincoll = Caml_state->stat_minor_collections;
+  intnat majcoll = Caml_state->stat_major_collections;
+  intnat heap_words = Caml_state->stat_heap_wsz;
+  intnat top_heap_words = Caml_state->stat_top_heap_wsz;
+  intnat cpct = Caml_state->stat_compactions;
+  intnat heap_chunks = Caml_state->stat_heap_chunks;
 
   res = caml_alloc_tuple (16);
   Store_field (res, 0, caml_copy_double (minwords));
@@ -321,8 +316,8 @@ CAMLprim value caml_gc_quick_stat(value v)
 
 double caml_gc_minor_words_unboxed()
 {
-  return (caml_stat_minor_words
-          + (double) (caml_young_alloc_end - caml_young_ptr));
+  return (Caml_state->stat_minor_words
+          + (double) (Caml_state->young_alloc_end - Caml_state->young_ptr));
 }
 
 CAMLprim value caml_gc_minor_words(value v)
@@ -337,10 +332,12 @@ CAMLprim value caml_gc_counters(value v)
   CAMLlocal1 (res);
 
   /* get a copy of these before allocating anything... */
-  double minwords = caml_stat_minor_words
-                    + (double) (caml_young_alloc_end - caml_young_ptr);
-  double prowords = caml_stat_promoted_words;
-  double majwords = caml_stat_major_words + (double) caml_allocated_words;
+  double minwords =
+    Caml_state->stat_minor_words
+    + (double) (Caml_state->young_alloc_end - Caml_state->young_ptr);
+  double prowords = Caml_state->stat_promoted_words;
+  double majwords =
+    Caml_state->stat_major_words + (double) caml_allocated_words;
 
   res = caml_alloc_tuple (3);
   Store_field (res, 0, caml_copy_double (minwords));
@@ -360,7 +357,7 @@ CAMLprim value caml_gc_get(value v)
   CAMLlocal1 (res);
 
   res = caml_alloc_tuple (11);
-  Store_field (res, 0, Val_long (caml_minor_heap_wsz));                 /* s */
+  Store_field (res, 0, Val_long (Caml_state->minor_heap_wsz));          /* s */
   Store_field (res, 1, Val_long (caml_major_heap_increment));           /* i */
   Store_field (res, 2, Val_long (caml_percent_free));                   /* o */
   Store_field (res, 3, Val_long (caml_verb_gc));                        /* v */
@@ -392,8 +389,13 @@ static uintnat norm_pmax (uintnat p)
 
 static intnat norm_minsize (intnat s)
 {
+  intnat page_wsize = Wsize_bsize(Page_size);
   if (s < Minor_heap_min) s = Minor_heap_min;
   if (s > Minor_heap_max) s = Minor_heap_max;
+  /* PR#9128 : Make sure the minor heap occupies an integral number of
+     pages, so that no page contains both bytecode and OCaml
+     values. This would confuse, e.g., caml_hash. */
+  s = (s + page_wsize - 1) / page_wsize * page_wsize;
   return s;
 }
 
@@ -419,9 +421,9 @@ CAMLprim value caml_gc_set(value v)
   uintnat newpf, newpm;
   asize_t newheapincr;
   asize_t newminwsz;
-  uintnat oldpolicy;
+  uintnat newpolicy;
   uintnat new_custom_maj, new_custom_min, new_custom_sz;
-  CAML_INSTR_SETUP (tmr, "");
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_SET);
 
   caml_verb_gc = Long_val (Field (v, 3));
 
@@ -456,12 +458,6 @@ CAMLprim value caml_gc_set(value v)
                        caml_major_heap_increment);
     }
   }
-  oldpolicy = caml_allocation_policy;
-  caml_set_allocation_policy (Long_val (Field (v, 6)));
-  if (oldpolicy != caml_allocation_policy){
-    caml_gc_message (0x20, "New allocation policy: %"
-                     ARCH_INTNAT_PRINTF_FORMAT "u\n", caml_allocation_policy);
-  }
 
   /* This field was added in 4.03.0. */
   if (Wosize_val (v) >= 8){
@@ -498,25 +494,46 @@ CAMLprim value caml_gc_set(value v)
     }
   }
 
-    /* Minor heap size comes last because it will trigger a minor collection
-       (thus invalidating [v]) and it can raise [Out_of_memory]. */
+  /* Save field 0 before [v] is invalidated. */
   newminwsz = norm_minsize (Long_val (Field (v, 0)));
-  if (newminwsz != caml_minor_heap_wsz){
+
+  /* Switching allocation policies must trigger a compaction, so it
+     invalidates [v]. */
+  newpolicy = Long_val (Field (v, 6));
+  if (newpolicy != caml_allocation_policy){
+    caml_empty_minor_heap ();
+    caml_finish_major_cycle ();
+    caml_finish_major_cycle ();
+    caml_compact_heap (newpolicy);
+    caml_gc_message (0x20, "New allocation policy: %"
+                     ARCH_INTNAT_PRINTF_FORMAT "u\n", newpolicy);
+  }
+
+  /* Minor heap size comes last because it can raise [Out_of_memory]. */
+  if (newminwsz != Caml_state->minor_heap_wsz){
     caml_gc_message (0x20, "New minor heap size: %"
                      ARCH_SIZET_PRINTF_FORMAT "uk words\n", newminwsz / 1024);
     caml_set_minor_heap_size (Bsize_wsize (newminwsz));
   }
-  CAML_INSTR_TIME (tmr, "explicit/gc_set");
+  CAML_EV_END(EV_EXPLICIT_GC_SET);
+
+  /* The compaction may have triggered some finalizers that we need to call. */
+  caml_process_pending_actions();
+
   return Val_unit;
 }
 
 CAMLprim value caml_gc_minor(value v)
 {
-  CAML_INSTR_SETUP (tmr, "");
+  value exn;
+
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_MINOR);
   CAMLassert (v == Val_unit);
   caml_request_minor_gc ();
-  caml_gc_dispatch ();
-  CAML_INSTR_TIME (tmr, "explicit/gc_minor");
+  // call the gc and call finalisers
+  exn = caml_process_pending_actions_exn();
+  CAML_EV_END(EV_EXPLICIT_GC_MINOR);
+  caml_raise_if_exception(exn);
   return Val_unit;
 }
 
@@ -524,74 +541,95 @@ static void test_and_compact (void)
 {
   double fp;
 
-  fp = 100.0 * caml_fl_cur_wsz / (caml_stat_heap_wsz - caml_fl_cur_wsz);
+  fp = 100.0 * caml_fl_cur_wsz / (Caml_state->stat_heap_wsz - caml_fl_cur_wsz);
   if (fp > 999999.0) fp = 999999.0;
   caml_gc_message (0x200, "Estimated overhead (lower bound) = %"
                           ARCH_INTNAT_PRINTF_FORMAT "u%%\n",
                    (uintnat) fp);
   if (fp >= caml_percent_max){
     caml_gc_message (0x200, "Automatic compaction triggered.\n");
-    caml_compact_heap ();
+    caml_compact_heap (-1);
   }
 }
 
 CAMLprim value caml_gc_major(value v)
 {
-  CAML_INSTR_SETUP (tmr, "");
+  value exn;
+
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_MAJOR);
   CAMLassert (v == Val_unit);
   caml_gc_message (0x1, "Major GC cycle requested\n");
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
   test_and_compact ();
-  caml_final_do_calls ();
-  CAML_INSTR_TIME (tmr, "explicit/gc_major");
+  // call finalisers
+  exn = caml_process_pending_actions_exn();
+  CAML_EV_END(EV_EXPLICIT_GC_MAJOR);
+  caml_raise_if_exception(exn);
   return Val_unit;
 }
 
 CAMLprim value caml_gc_full_major(value v)
 {
-  CAML_INSTR_SETUP (tmr, "");
+  value exn;
+
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_FULL_MAJOR);
   CAMLassert (v == Val_unit);
   caml_gc_message (0x1, "Full major GC cycle requested\n");
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
-  caml_final_do_calls ();
+  // call finalisers
+  exn = caml_process_pending_actions_exn();
+  if (Is_exception_result(exn)) goto cleanup;
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
   test_and_compact ();
-  caml_final_do_calls ();
-  CAML_INSTR_TIME (tmr, "explicit/gc_full_major");
+  // call finalisers
+  exn = caml_process_pending_actions_exn();
+
+cleanup:
+  CAML_EV_END(EV_EXPLICIT_GC_FULL_MAJOR);
+  caml_raise_if_exception(exn);
+
   return Val_unit;
 }
 
 CAMLprim value caml_gc_major_slice (value v)
 {
-  CAML_INSTR_SETUP (tmr, "");
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_MAJOR_SLICE);
   CAMLassert (Is_long (v));
   caml_major_collection_slice (Long_val (v));
-  CAML_INSTR_TIME (tmr, "explicit/gc_major_slice");
+  CAML_EV_END(EV_EXPLICIT_GC_MAJOR_SLICE);
   return Val_long (0);
 }
 
 CAMLprim value caml_gc_compaction(value v)
 {
-  CAML_INSTR_SETUP (tmr, "");
+  value exn;
+
+  CAML_EV_BEGIN(EV_EXPLICIT_GC_COMPACT);
   CAMLassert (v == Val_unit);
   caml_gc_message (0x10, "Heap compaction requested\n");
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
-  caml_final_do_calls ();
+  // call finalisers
+  exn = caml_process_pending_actions_exn();
+  if (Is_exception_result(exn)) goto cleanup;
   caml_empty_minor_heap ();
   caml_finish_major_cycle ();
-  caml_compact_heap ();
-  caml_final_do_calls ();
-  CAML_INSTR_TIME (tmr, "explicit/gc_compact");
+  caml_compact_heap (-1);
+  // call finalisers
+  exn = caml_process_pending_actions_exn();
+
+ cleanup:
+  CAML_EV_END(EV_EXPLICIT_GC_COMPACT);
+  caml_raise_if_exception(exn);
   return Val_unit;
 }
 
 CAMLprim value caml_get_minor_free (value v)
 {
-  return Val_int (caml_young_ptr - caml_young_alloc_start);
+  return Val_int (Caml_state->young_ptr - Caml_state->young_alloc_start);
 }
 
 CAMLprim value caml_get_major_bucket (value v)
@@ -614,14 +652,6 @@ CAMLprim value caml_get_major_credit (value v)
   return Val_long ((long) (caml_major_work_credit * 1e6));
 }
 
-uintnat caml_normalize_heap_increment (uintnat i)
-{
-  if (i < Bsize_wsize (Heap_chunk_min)){
-    i = Bsize_wsize (Heap_chunk_min);
-  }
-  return ((i + Page_size - 1) >> Page_log) << Page_log;
-}
-
 /* [minor_size] and [major_size] are numbers of words
    [major_incr] is either a percentage or a number of words */
 void caml_init_gc (uintnat minor_size, uintnat major_size,
@@ -630,31 +660,29 @@ void caml_init_gc (uintnat minor_size, uintnat major_size,
                    uintnat custom_maj, uintnat custom_min,
                    uintnat custom_bsz)
 {
-  uintnat major_heap_size =
-    Bsize_wsize (caml_normalize_heap_increment (major_size));
+  uintnat major_bsize;
+  if (major_size < Heap_chunk_min) major_size = Heap_chunk_min;
+  major_bsize = Bsize_wsize(major_size);
+  major_bsize = ((major_bsize + Page_size - 1) >> Page_log) << Page_log;
 
-  caml_instr_init ();
-  if (caml_init_alloc_for_heap () != 0){
-    caml_fatal_error ("cannot initialize heap: mmap failed");
-  }
-  if (caml_page_table_initialize(Bsize_wsize(minor_size) + major_heap_size)){
+  if (caml_page_table_initialize(Bsize_wsize(minor_size) + major_bsize)){
     caml_fatal_error ("cannot initialize page table");
   }
   caml_set_minor_heap_size (Bsize_wsize (norm_minsize (minor_size)));
   caml_major_heap_increment = major_incr;
   caml_percent_free = norm_pfree (percent_fr);
   caml_percent_max = norm_pmax (percent_m);
-  caml_init_major_heap (major_heap_size);
+  caml_init_major_heap (major_bsize);
   caml_major_window = norm_window (window);
   caml_custom_major_ratio = norm_custom_maj (custom_maj);
   caml_custom_minor_ratio = norm_custom_min (custom_min);
   caml_custom_minor_max_bsz = custom_bsz;
   caml_gc_message (0x20, "Initial minor heap size: %"
                    ARCH_SIZET_PRINTF_FORMAT "uk words\n",
-                   caml_minor_heap_wsz / 1024);
+                   Caml_state->minor_heap_wsz / 1024);
   caml_gc_message (0x20, "Initial major heap size: %"
                    ARCH_INTNAT_PRINTF_FORMAT "uk bytes\n",
-                   major_heap_size / 1024);
+                   major_bsize / 1024);
   caml_gc_message (0x20, "Initial space overhead: %"
                    ARCH_INTNAT_PRINTF_FORMAT "u%%\n", caml_percent_free);
   caml_gc_message (0x20, "Initial max overhead: %"
@@ -701,7 +729,7 @@ CAMLprim value caml_runtime_parameters (value unit)
     ("a=%d,b=%d,H=%"F_Z"u,i=%"F_Z"u,l=%"F_Z"u,o=%"F_Z"u,O=%"F_Z"u,p=%d,"
      "s=%"F_S"u,t=%"F_Z"u,v=%"F_Z"u,w=%d,W=%"F_Z"u",
      /* a */ (int) caml_allocation_policy,
-     /* b */ caml_backtrace_active,
+     /* b */ (int) Caml_state->backtrace_active,
      /* h */ /* missing */ /* FIXME add when changed to min_heap_size */
      /* H */ caml_use_huge_pages,
      /* i */ caml_major_heap_increment,
@@ -714,7 +742,7 @@ CAMLprim value caml_runtime_parameters (value unit)
      /* O */ caml_percent_max,
      /* p */ caml_parser_trace,
      /* R */ /* missing */
-     /* s */ caml_minor_heap_wsz,
+     /* s */ Caml_state->minor_heap_wsz,
      /* t */ caml_trace_level,
      /* v */ caml_verb_gc,
      /* w */ caml_major_window,
