@@ -31,9 +31,12 @@
 #include "caml/io.h"
 #include "caml/md5.h"
 #include "caml/memory.h"
+#include "caml/memprof.h"
 #include "caml/mlvalues.h"
 #include "caml/misc.h"
 #include "caml/reverse.h"
+#include "caml/signals.h"
+
 
 static unsigned char * intern_src;
 /* Reading pointer in block holding input data. */
@@ -74,27 +77,27 @@ CAMLnoreturn_end;
 
 static void intern_free_stack(void);
 
-static inline unsigned char read8u(void)
+Caml_inline unsigned char read8u(void)
 { return *intern_src++; }
 
-static inline signed char read8s(void)
+Caml_inline signed char read8s(void)
 { return *intern_src++; }
 
-static inline uint16_t read16u(void)
+Caml_inline uint16_t read16u(void)
 {
   uint16_t res = (intern_src[0] << 8) + intern_src[1];
   intern_src += 2;
   return res;
 }
 
-static inline int16_t read16s(void)
+Caml_inline int16_t read16s(void)
 {
   int16_t res = (intern_src[0] << 8) + intern_src[1];
   intern_src += 2;
   return res;
 }
 
-static inline uint32_t read32u(void)
+Caml_inline uint32_t read32u(void)
 {
   uint32_t res =
     ((uint32_t)(intern_src[0]) << 24) + (intern_src[1] << 16)
@@ -103,7 +106,7 @@ static inline uint32_t read32u(void)
   return res;
 }
 
-static inline int32_t read32s(void)
+Caml_inline int32_t read32s(void)
 {
   int32_t res =
     ((uint32_t)(intern_src[0]) << 24) + (intern_src[1] << 16)
@@ -129,7 +132,7 @@ static uintnat read64u(void)
 }
 #endif
 
-static inline void readblock(void * dest, intnat len)
+Caml_inline void readblock(void * dest, intnat len)
 {
   memcpy(dest, intern_src, len);
   intern_src += len;
@@ -505,7 +508,7 @@ static void intern_rec(value *dest)
         if (codeptr != NULL) {
           v = (value) codeptr;
         } else {
-          value * function_placeholder =
+          const value * function_placeholder =
             caml_named_value ("Debugger.function_placeholder");
           if (function_placeholder != NULL) {
             v = *function_placeholder;
@@ -573,7 +576,7 @@ static void intern_rec(value *dest)
 
         if (ops->finalize != NULL && Is_young(v)) {
           /* Remember that the block has a finalizer. */
-          add_to_custom_table (&caml_custom_table, v, 0, 1);
+          add_to_custom_table (Caml_state->custom_table, v, 0, 1);
         }
 
         intern_dest += 1 + size;
@@ -625,11 +628,15 @@ static void intern_alloc(mlsize_t whsize, mlsize_t num_objects,
     if (wosize <= Max_young_wosize){
       if (wosize == 0){
         intern_block = Atom (String_tag);
-      } else {
-        intern_block = caml_alloc_small (wosize, String_tag);
+      }else{
+#define Setup_for_gc
+#define Restore_after_gc
+        Alloc_small_no_track(intern_block, wosize, String_tag);
+#undef Setup_for_gc
+#undef Restore_after_gc
       }
     }else{
-      intern_block = caml_alloc_shr_no_raise (wosize, String_tag);
+      intern_block = caml_alloc_shr_no_track_noexc (wosize, String_tag);
       /* do not do the urgent_gc check here because it might darken
          intern_block into gray and break the intern_color assertion below */
       if (intern_block == 0) {
@@ -655,8 +662,9 @@ static void intern_alloc(mlsize_t whsize, mlsize_t num_objects,
     CAMLassert(intern_obj_table == NULL);
 }
 
-static void intern_add_to_heap(mlsize_t whsize)
+static header_t* intern_add_to_heap(mlsize_t whsize)
 {
+  header_t* res = NULL;
   /* Add new heap chunk to heap if needed */
   if (intern_extra_block != NULL) {
     /* If heap chunk not filled totally, build free block at end */
@@ -671,11 +679,37 @@ static void intern_add_to_heap(mlsize_t whsize)
     }
     caml_allocated_words +=
       Wsize_bsize ((char *) intern_dest - intern_extra_block);
-    caml_add_to_heap(intern_extra_block);
+    if(caml_add_to_heap(intern_extra_block) != 0) {
+      intern_cleanup();
+      caml_raise_out_of_memory();
+    }
+    res = (header_t*)intern_extra_block;
     intern_extra_block = NULL; // To prevent intern_cleanup freeing it
-  } else {
+  } else if(intern_block != 0) { /* [intern_block = 0] when [whsize = 0]  */
+    res = Hp_val(intern_block);
     intern_block = 0; // To prevent intern_cleanup rewriting its header
   }
+  return res;
+}
+
+static value intern_end(value res, mlsize_t whsize)
+{
+  CAMLparam1(res);
+  header_t *block = intern_add_to_heap(whsize);
+  header_t *blockend = intern_dest;
+
+  /* Free everything */
+  intern_cleanup();
+
+  /* Memprof tracking has to be done here, because unmarshalling can
+     still fail until now. */
+  if(block != NULL)
+    caml_memprof_track_interned(block, blockend);
+
+  // Give gc a chance to run, and run memprof callbacks
+  caml_process_pending_actions();
+
+  CAMLreturn(res);
 }
 
 /* Parsing the header */
@@ -772,16 +806,16 @@ static value caml_input_val_core(struct channel *chan, int outside_heap)
   intern_alloc(h.whsize, h.num_objects, outside_heap);
   /* Fill it in */
   intern_rec(&res);
-  if (!outside_heap) {
-    intern_add_to_heap(h.whsize);
-  } else {
+  if (!outside_heap)
+    return intern_end(res, h.whsize);
+  else {
     caml_disown_for_heap(intern_extra_block);
     intern_extra_block = NULL;
     intern_block = 0;
+    /* Free everything */
+    intern_cleanup();
+    return caml_check_urgent_gc(res);
   }
-  /* Free everything */
-  intern_cleanup();
-  return caml_check_urgent_gc(res);
 }
 
 value caml_input_val(struct channel* chan)
@@ -831,10 +865,7 @@ CAMLexport value caml_input_val_from_bytes(value str, intnat ofs)
   intern_src = &Byte_u(str, ofs + h.header_len); /* If a GC occurred */
   /* Fill it in */
   intern_rec(&obj);
-  intern_add_to_heap(h.whsize);
-  /* Free everything */
-  intern_cleanup();
-  CAMLreturn (caml_check_urgent_gc(obj));
+  CAMLreturn (intern_end(obj, h.whsize));
 }
 
 CAMLprim value caml_input_value_from_string(value str, value ofs)
@@ -854,10 +885,7 @@ static value input_val_from_block(struct marshal_header * h)
   intern_alloc(h->whsize, h->num_objects, 0);
   /* Fill it in */
   intern_rec(&obj);
-  intern_add_to_heap(h->whsize);
-  /* Free internal data structures */
-  intern_cleanup();
-  return caml_check_urgent_gc(obj);
+  return (intern_end(obj, h->whsize));
 }
 
 CAMLexport value caml_input_value_from_malloc(char * data, intnat ofs)
