@@ -28,8 +28,8 @@
        struct
          type t = int * int
          let compare (x0,y0) (x1,y1) =
-           match Pervasives.compare x0 x1 with
-               0 -> Pervasives.compare y0 y1
+           match Stdlib.compare x0 x1 with
+               0 -> Stdlib.compare y0 y1
              | c -> c
        end
 
@@ -55,7 +55,7 @@ module type OrderedType =
           [f e1 e2] is strictly negative if [e1] is smaller than [e2],
           and [f e1 e2] is strictly positive if [e1] is greater than [e2].
           Example: a suitable ordering function is the generic structural
-          comparison function {!Pervasives.compare}. *)
+          comparison function {!Stdlib.compare}. *)
   end
 (** Input signature of the functor {!Map.Make}. *)
 
@@ -86,6 +86,19 @@ module type S =
        of [x] in [m] disappears.
        @before 4.03 Physical equality was not ensured. *)
 
+    val update: key -> ('a option -> 'a option) -> 'a t -> 'a t
+    (** [update x f m] returns a map containing the same bindings as
+        [m], except for the binding of [x]. Depending on the value of
+        [y] where [y] is [f (find_opt x m)], the binding of [x] is
+        added, removed or updated. If [y] is [None], the binding is
+        removed if it exists; otherwise, if [y] is [Some z] then [x]
+        is associated to [z] in the resulting map.  If [x] was already
+        bound in [m] to a value that is physically equal to [z], [m]
+        is returned unchanged (the result of the function is then
+        physically equal to [m]).
+        @since 4.06.0
+    *)
+
     val singleton: key -> 'a -> 'a t
     (** [singleton x y] returns the one-element map that contains a binding [y]
         for [x].
@@ -101,25 +114,25 @@ module type S =
 
     val merge:
          (key -> 'a option -> 'b option -> 'c option) -> 'a t -> 'b t -> 'c t
-    (** [merge f m1 m2] computes a map whose keys is a subset of keys of [m1]
-        and of [m2]. The presence of each such binding, and the corresponding
-        value, is determined with the function [f].
+    (** [merge f m1 m2] computes a map whose keys are a subset of the keys of
+        [m1] and of [m2]. The presence of each such binding, and the
+        corresponding value, is determined with the function [f].
         In terms of the [find_opt] operation, we have
-        [find_opt x (merge f m1 m2) = f (find_opt x m1) (find_opt x m2)]
-        for any key [x], provided that [f None None = None].
+        [find_opt x (merge f m1 m2) = f x (find_opt x m1) (find_opt x m2)]
+        for any key [x], provided that [f x None None = None].
         @since 3.12.0
      *)
 
     val union: (key -> 'a -> 'a -> 'a option) -> 'a t -> 'a t -> 'a t
-    (** [union f m1 m2] computes a map whose keys is the union of keys
+    (** [union f m1 m2] computes a map whose keys are a subset of the keys
         of [m1] and of [m2].  When the same binding is defined in both
         arguments, the function [f] is used to combine them.
         This is a special case of [merge]: [union f m1 m2] is equivalent
         to [merge f' m1 m2], where
-        - [f' None None = None]
-        - [f' (Some v) None = Some v]
-        - [f' None (Some v) = Some v]
-        - [f' (Some v1) (Some v2) = f v1 v2]
+        - [f' _key None None = None]
+        - [f' _key (Some v) None = Some v]
+        - [f' _key None (Some v) = Some v]
+        - [f' key (Some v1) (Some v2) = f key v1 v2]
 
         @since 4.03.0
     *)
@@ -159,18 +172,38 @@ module type S =
 
     val filter: (key -> 'a -> bool) -> 'a t -> 'a t
     (** [filter p m] returns the map with all the bindings in [m]
-        that satisfy predicate [p]. If [p] satisfies every binding in [m],
+        that satisfy predicate [p]. If every binding in [m] satisfies [p],
         [m] is returned unchanged (the result of the function is then
         physically equal to [m])
         @since 3.12.0
        @before 4.03 Physical equality was not ensured.
      *)
 
+    val filter_map: (key -> 'a -> 'b option) -> 'a t -> 'b t
+    (** [filter_map f m] applies the function [f] to every binding of
+        [m], and builds a map from the results. For each binding
+        [(k, v)] in the input map:
+        - if [f k v] is [None] then [k] is not in the result,
+        - if [f k v] is [Some v'] then the binding [(k, v')]
+          is in the output map.
+
+        For example, the following function on maps whose values are lists
+        {[
+        filter_map
+          (fun _k li -> match li with [] -> None | _::tl -> Some tl)
+          m
+        ]}
+        drops all bindings of [m] whose value is an empty list, and pops
+        the first element of each value that is non-empty.
+
+        @since 4.11.0
+     *)
+
     val partition: (key -> 'a -> bool) -> 'a t -> 'a t * 'a t
     (** [partition p m] returns a pair of maps [(m1, m2)], where
-        [m1] contains all the bindings of [s] that satisfy the
+        [m1] contains all the bindings of [m] that satisfy the
         predicate [p], and [m2] is the map with all the bindings of
-        [s] that do not satisfy [p].
+        [m] that do not satisfy [p].
         @since 3.12.0
      *)
 
@@ -181,35 +214,35 @@ module type S =
 
     val bindings: 'a t -> (key * 'a) list
     (** Return the list of all bindings of the given map.
-       The returned list is sorted in increasing order with respect
+       The returned list is sorted in increasing order of keys with respect
        to the ordering [Ord.compare], where [Ord] is the argument
        given to {!Map.Make}.
         @since 3.12.0
      *)
 
     val min_binding: 'a t -> (key * 'a)
-    (** Return the smallest binding of the given map
+    (** Return the binding with the smallest key in a given map
        (with respect to the [Ord.compare] ordering), or raise
        [Not_found] if the map is empty.
         @since 3.12.0
      *)
 
     val min_binding_opt: 'a t -> (key * 'a) option
-    (** Return the smallest binding of the given map
+    (** Return the binding with the smallest key in the given map
        (with respect to the [Ord.compare] ordering), or [None]
        if the map is empty.
         @since 4.05
      *)
 
     val max_binding: 'a t -> (key * 'a)
-    (** Same as {!Map.S.min_binding}, but returns the largest binding
-        of the given map.
+    (** Same as {!Map.S.min_binding}, but returns the binding with
+        the largest key in the given map.
         @since 3.12.0
      *)
 
     val max_binding_opt: 'a t -> (key * 'a) option
-    (** Same as {!Map.S.min_binding_opt}, but returns the largest binding
-        of the given map.
+    (** Same as {!Map.S.min_binding_opt}, but returns the binding with
+        the largest key in the given map.
         @since 4.05
      *)
 
@@ -239,12 +272,12 @@ module type S =
      *)
 
     val find: key -> 'a t -> 'a
-    (** [find x m] returns the current binding of [x] in [m],
-       or raises [Not_found] if no such binding exists. *)
+    (** [find x m] returns the current value of [x] in [m],
+       or raises [Not_found] if no binding for [x] exists. *)
 
     val find_opt: key -> 'a t -> 'a option
-    (** [find_opt x m] returns [Some v] if the current binding of [x]
-        in [m] is [v], or [None] if no such binding exists.
+    (** [find_opt x m] returns [Some v] if the current value of [x]
+        in [m] is [v], or [None] if no binding for [x] exists.
         @since 4.05
     *)
 
@@ -293,7 +326,24 @@ module type S =
     (** Same as {!Map.S.map}, but the function receives as arguments both the
        key and the associated value for each binding of the map. *)
 
+    (** {1 Iterators} *)
 
+    val to_seq : 'a t -> (key * 'a) Seq.t
+    (** Iterate on the whole map, in ascending order of keys
+        @since 4.07 *)
+
+    val to_seq_from : key -> 'a t -> (key * 'a) Seq.t
+    (** [to_seq_from k m] iterates on a subset of the bindings of [m],
+        in ascending order of keys, from key [k] or above.
+        @since 4.07 *)
+
+    val add_seq : (key * 'a) Seq.t -> 'a t -> 'a t
+    (** Add the given bindings to the map, in order.
+        @since 4.07 *)
+
+    val of_seq : (key * 'a) Seq.t -> 'a t
+    (** Build a map from the given bindings
+        @since 4.07 *)
   end
 (** Output signature of the functor {!Map.Make}. *)
 
