@@ -1,3 +1,6 @@
+(* TEST
+  arguments = "wc.ml"
+*)
 
 (* Counts characters, lines and words in one or several files. *)
 
