@@ -1,3 +1,7 @@
+(* TEST
+   include testing
+*)
+
 let f x = Sys.opaque_identity x
 
 let () =
