@@ -1,4 +1,6 @@
 (* TEST
+   * native
+     compare_programs = "false"
 *)
 
 (** Test that the right message errors are emitted by Arg *)
