@@ -1,3 +1,33 @@
+(* TEST
+
+files = "bigarrf.f bigarrfstub.c"
+last_flags = "-cclib -lgfortran"
+
+* script
+script = "sh ${test_source_directory}/has-gfortran.sh"
+
+** setup-ocamlc.byte-build-env
+*** script
+script = "gfortran -c bigarrf.f"
+**** ocamlc.byte
+all_modules = "bigarrf.o bigarrfstub.c bigarrfml.ml"
+***** run
+output = "${test_build_directory}/program-output"
+stdout = "${output}"
+****** check-program-output
+
+** setup-ocamlopt.byte-build-env
+*** script
+script = "gfortran -c bigarrf.f"
+**** ocamlopt.byte
+all_modules = "bigarrf.o bigarrfstub.c bigarrfml.ml"
+***** run
+output = "${test_build_directory}/program-output"
+stdout = "${output}"
+****** check-program-output
+
+*)
+
 open Bigarray
 open Printf
 
