@@ -1,3 +1,6 @@
+(* TEST
+*)
+
 open Bigarray
 open Printf
 open Complex
@@ -976,61 +979,6 @@ let tests () =
   test_structured_io 14 (make_array3 complex64 fortran_layout 1 10 20 30
                                      makecomplex);
 
-  testing_function "map_file";
-  let mapped_file = Filename.temp_file "bigarray" ".data" in
-  begin
-    let fd =
-     Unix.openfile mapped_file
-                   [Unix.O_RDWR; Unix.O_TRUNC; Unix.O_CREAT] 0o666 in
-    let a = Array1.map_file fd float64 c_layout true 10000 in
-    Unix.close fd;
-    for i = 0 to 9999 do a.{i} <- float i done;
-    let fd = Unix.openfile mapped_file [Unix.O_RDONLY] 0 in
-    let b = Array2.map_file fd float64 fortran_layout false 100 (-1) in
-    Unix.close fd;
-    let ok = ref true in
-    for i = 0 to 99 do
-      for j = 0 to 99 do
-        if b.{j+1,i+1} <> float (100 * i + j) then ok := false
-      done
-    done;
-    test 1 !ok true;
-    b.{50,50} <- (-1.0);
-    let fd = Unix.openfile mapped_file [Unix.O_RDONLY] 0 in
-    let c = Array2.map_file fd float64 c_layout false (-1) 100 in
-    Unix.close fd;
-    let ok = ref true in
-    for i = 0 to 99 do
-      for j = 0 to 99 do
-        if c.{i,j} <> float (100 * i + j) then ok := false
-      done
-    done;
-    test 2 !ok true;
-    let fd = Unix.openfile mapped_file [Unix.O_RDONLY] 0 in
-    let c = Array2.map_file fd ~pos:800L float64 c_layout false (-1) 100 in
-    Unix.close fd;
-    let ok = ref true in
-    for i = 1 to 99 do
-      for j = 0 to 99 do
-        if c.{i-1,j} <> float (100 * i + j) then ok := false
-      done
-    done;
-    test 3 !ok true;
-    let fd = Unix.openfile mapped_file [Unix.O_RDONLY] 0 in
-    let c = Array2.map_file fd ~pos:79200L float64 c_layout false (-1) 100 in
-    Unix.close fd;
-    let ok = ref true in
-    for j = 0 to 99 do
-      if c.{0,j} <> float (100 * 99 + j) then ok := false
-    done;
-    test 4 !ok true
-  end;
-  (* Force garbage collection of the mapped bigarrays above, otherwise
-     Win32 doesn't let us erase the file.  Notice the begin...end above
-     so that the VM doesn't keep stack references to the mapped bigarrays. *)
-  Gc.full_major();
-  Sys.remove mapped_file;
-
   ()
   [@@inline never]
 
