@@ -10,6 +10,7 @@ let string_of_even_opt x =
 (* Standard test case *)
 let () =
   let l = List.init 10 (fun x -> x) in
+  let sl = List.init 10 string_of_int in
   assert (List.exists (fun a -> a < 10) l);
   assert (List.exists (fun a -> a > 0) l);
   assert (List.exists (fun a -> a = 0) l);
@@ -26,6 +27,15 @@ let () =
   assert (not (List.exists (fun a -> a > 9) l));
   assert (List.exists (fun _ -> true) l);
 
+  begin
+    let f ~limit a = if a >= limit then Some (a, limit) else None in
+    assert (List.find_map (f ~limit:3) [] = None);
+    assert (List.find_map (f ~limit:3) l = Some (3, 3));
+    assert (List.find_map (f ~limit:30) l = None);
+  end;
+
+  assert (List.filteri (fun i _ -> i < 2) (List.rev l) = [9; 8]);
+
   assert (List.compare_lengths [] [] = 0);
   assert (List.compare_lengths [1;2] ['a';'b'] = 0);
   assert (List.compare_lengths [] [1;2] < 0);
@@ -42,6 +52,15 @@ let () =
   assert (List.compare_length_with ['1'] 1 = 0);
   assert (List.compare_length_with ['1'] 2 < 0);
   assert (List.filter_map string_of_even_opt l = ["0";"2";"4";"6";"8"]);
+  assert (List.concat_map (fun i -> [i; i+1]) [1; 5] = [1; 2; 5; 6]);
+  assert (
+    let count = ref 0 in
+    List.concat_map (fun i -> incr count; [i; !count]) [1; 5] = [1; 1; 5; 2]);
+  assert (List.fold_left_map (fun a b -> a + b, b) 0 l = (45, l));
+  assert (List.fold_left_map (fun a b -> assert false) 0 [] = (0, []));
+  assert (
+    let f a b = a + b, string_of_int b in
+    List.fold_left_map f 0 l = (45, sl));
   ()
 ;;
 
