@@ -15,6 +15,7 @@
 
 #include <caml/mlvalues.h>
 #include <caml/intext.h>
+#include <caml/custom.h>
 
 #define CAML_INTERNALS
 
@@ -28,3 +29,36 @@ value marshal_from_block(value vbuf, value vlen)
 {
   return caml_input_value_from_block((char *) vbuf, Long_val(vlen));
 }
+
+static void bad_serialize(value v, uintnat* sz_32, uintnat* sz_64)
+{
+  caml_serialize_int_4(42);
+  *sz_32 = *sz_64 = 100;
+}
+
+static uintnat bad_deserialize(void* dst)
+{
+  return 10;
+}
+
+static struct custom_operations buggy_ops = {
+  "foo",
+  custom_finalize_default,
+  custom_compare_default,
+  custom_hash_default,
+  bad_serialize,
+  bad_deserialize,
+  custom_compare_ext_default,
+  custom_fixed_length_default
+};
+
+value init_buggy_custom_ops()
+{
+  caml_register_custom_operations(&buggy_ops);
+  return Val_unit;
+}
+
+value value_with_buggy_serialiser()
+{
+  return caml_alloc_custom(&buggy_ops, 20, 0, 1);
+}
