@@ -1,14 +1,7 @@
-(*************************************************************************)
-(*                                                                       *)
-(*                                 OCaml                                 *)
-(*                                                                       *)
-(*            Pierre Weis, projet Cristal, INRIA Rocquencourt            *)
-(*                                                                       *)
-(*   Copyright 2002 Institut National de Recherche en Informatique et    *)
-(*   en Automatique.  All rights reserved.  This file is distributed     *)
-(*   under the terms of the Q Public License version 1.0.                *)
-(*                                                                       *)
-(*************************************************************************)
+(* TEST
+   include testing
+   compare_programs = "false" (* See https://github.com/ocaml/ocaml/pull/8853 *)
+*)
 
 (*
 
@@ -104,6 +97,12 @@ let test4 () =
 test (test4 ())
 ;;
 
+let same_float x y =
+  let is_nan z = (z <> z) in
+  if is_nan x then is_nan y
+  else Int64.bits_of_float y = Int64.bits_of_float x
+;;
+
 (* e style. *)
 let test5 () =
   bscanf (Scanning.from_string "1e1")
@@ -122,6 +121,58 @@ let test5 () =
     "%g %g %g %g"
     (fun b1 b2 b3 b4 ->
      b1 = 1.0 && b2 = 1.1 && b3 = 0.0 && b4 = 0.13)
+  &&
+(* F style *)
+  bscanf (Scanning.from_string "1.5 1.5e0 15e-1 0x1.8 0X1.8")
+    "%F %F %f %F %F"
+    (fun b1 b2 b3 b4 b5 -> b1 = b2 && b2 = b3 && b3 = b4 && b4 = b5)
+  &&
+(* h style *)
+  begin
+    let roundtrip x =
+      bscanf (Printf.ksprintf Scanning.from_string "%h" x) "%h" (same_float x)
+    in
+    roundtrip (+0.) &&
+    roundtrip (-0.) &&
+    roundtrip (+1.) &&
+    roundtrip (-1.) &&
+    roundtrip (+1024.) &&
+    roundtrip (-1024.) &&
+    roundtrip 0X123.456 &&
+    roundtrip 0X123456789ABCDE. &&
+    roundtrip epsilon_float &&
+    roundtrip (4. *. atan 1.) &&
+    (Sys.win32 ||
+     (* nan/infinity parsing fails on Windows? *)
+     (roundtrip nan &&
+      roundtrip infinity &&
+      roundtrip neg_infinity)) &&
+    true
+  end
+  &&
+
+  (* H style *)
+  begin
+    let roundtrip x =
+      bscanf (Printf.ksprintf Scanning.from_string "%H" x) "%H" (same_float x)
+    in
+    roundtrip (+0.) &&
+    roundtrip (-0.) &&
+    roundtrip (+1.) &&
+    roundtrip (-1.) &&
+    roundtrip (+1024.) &&
+    roundtrip (-1024.) &&
+    roundtrip 0X123.456 &&
+    roundtrip 0X123456789ABCDE. &&
+    roundtrip epsilon_float &&
+    roundtrip (4. *. atan 1.) &&
+    (Sys.win32 ||
+     (* nan/infinity parsing fails on Windows? *)
+     (roundtrip nan &&
+      roundtrip infinity &&
+      roundtrip neg_infinity)) &&
+    true
+  end
 ;;
 
 test (test5 ())
@@ -404,7 +455,7 @@ let rec scan_elems ib accu =
   bscanf ib " %c " (fun c ->
     match c with
     | '[' when accu = [] ->
-        (* begginning of list: could find either
+        (* beginning of list: could find either
            - an int, if the list is not empty,
            - the char ], if the list is empty. *)
         bscanf ib "%[]]"
@@ -881,7 +932,7 @@ let scan_string_list = scan_list (fun ib -> Scanf.bscanf ib "%S");;
 let scan_bool_list = scan_list (fun ib -> Scanf.bscanf ib "%B");;
 let scan_char_list = scan_list (fun ib -> Scanf.bscanf ib "%C");;
 
-(* [scan_list] is truely polymorphic: scanning a list of lists of items
+(* [scan_list] is truly polymorphic: scanning a list of lists of items
    is a one liner!
 
    Here we scan list of lists of floats. *)
@@ -933,7 +984,7 @@ let test35 () =
 test (test340 () && test35 ())
 ;;
 
-(* The prefered reader functionnals. *)
+(* The preferred reader functionnals. *)
 
 (* To read a list as in OCaml (elements are ``blank + semicolon + blank''
    separated, and the list is enclosed in brackets). *)
@@ -1185,7 +1236,7 @@ let next_char ob () =
 
 let send_string ob s =
   Buffer.add_string ob s; Buffer.add_char ob '\n';;
-let send_int ob i = send_string ob (string_of_int i);;
+let send_int ob i = send_string ob (Int.to_string i);;
 
 let rec reader =
   let count = ref 0 in
@@ -1356,7 +1407,7 @@ test (test54 ())
 
 (* Creating digests for files. *)
 let add_digest_ib ob ib =
-  let digest s = String.uppercase (Digest.to_hex (Digest.string s)) in
+  let digest s = String.uppercase_ascii (Digest.to_hex (Digest.string s)) in
   let scan_line ib f = Scanf.bscanf ib "%[^\n\r]\n" f in
   let output_line_digest s =
     Buffer.add_string ob s;
@@ -1488,4 +1539,20 @@ let test60 () =
   sscanf "abc" "%1s%s" (fun s1 s2 -> s1 = "a" && s2 = "bc")
 ;;
 
-test (test60 ());
+test (test60 ())
+;;
+
+let test61 () =
+  let test fmt =
+    format_from_string (string_of_format fmt) fmt = fmt
+  in
+  test "%s/%a" &&
+  test "\\ " &&
+  test "\\x" &&
+  test "\\x25s" &&
+  test "\\\"%s" &&
+  test "\\"
+;;
+
+test (test61 ())
+;;
