@@ -1,6 +1,9 @@
 (* TEST
 
+* hassysthreads
 include systhreads
+** bytecode
+** native
 
 *)
 
