@@ -1,3 +1,11 @@
+(* TEST
+include tool-ocaml-lib
+flags = "-w a"
+ocaml_script_as_argument = "true"
+* setup-ocaml-build-env
+** ocaml
+*)
+
 open Lib;;
 match -1 with
 | 0 -> raise Not_found
@@ -7,25 +15,25 @@ match -1 with
 
 (**
        0 CONSTINT 42
-       2 PUSHACC0 
+       2 PUSHACC0
        3 MAKEBLOCK1 0
        5 POP 1
        7 SETGLOBAL Lib
        9 CONSTINT -1
-      11 PUSHACC0 
-      12 SWITCH 
+      11 PUSHACC0
+      12 SWITCH
         int 0 -> 18
         int 1 -> 23
       16 BRANCH 28
       18 GETGLOBAL Not_found
       20 MAKEBLOCK1 0
-      22 RAISE 
+      22 RAISE
       23 GETGLOBAL Not_found
       25 MAKEBLOCK1 0
-      27 RAISE 
-      28 CONST0 
+      27 RAISE
+      28 CONST0
       29 POP 1
-      31 ATOM0 
+      31 ATOM0
       32 SETGLOBAL T140-switch-4
-      34 STOP 
+      34 STOP
 **)
