@@ -1,3 +1,11 @@
+(* TEST
+include tool-ocaml-lib
+flags = "-w a"
+ocaml_script_as_argument = "true"
+* setup-ocaml-build-env
+** ocaml
+*)
+
 open Lib;;
 type t =
  | A
@@ -12,23 +20,23 @@ match A with
 
 (**
        0 CONSTINT 42
-       2 PUSHACC0 
+       2 PUSHACC0
        3 MAKEBLOCK1 0
        5 POP 1
        7 SETGLOBAL Lib
-       9 CONST0 
-      10 PUSHACC0 
-      11 SWITCH 
+       9 CONST0
+      10 PUSHACC0
+      11 SWITCH
         int 0 -> 16
         tag 0 -> 19
         tag 1 -> 19
-      16 CONST0 
+      16 CONST0
       17 BRANCH 24
       19 GETGLOBAL Not_found
       21 MAKEBLOCK1 0
-      23 RAISE 
+      23 RAISE
       24 POP 1
-      26 ATOM0 
+      26 ATOM0
       27 SETGLOBAL T142-switch-8
-      29 STOP 
+      29 STOP
 **)
