@@ -1,3 +1,11 @@
+(* TEST
+include tool-ocaml-lib
+flags = "-w a"
+ocaml_script_as_argument = "true"
+* setup-ocaml-build-env
+** ocaml
+*)
+
 open Lib;;
 type t =
  | A
@@ -12,23 +20,23 @@ match C 0 with
 
 (**
        0 CONSTINT 42
-       2 PUSHACC0 
+       2 PUSHACC0
        3 MAKEBLOCK1 0
        5 POP 1
        7 SETGLOBAL Lib
        9 GETGLOBAL <1>(0)
-      11 PUSHACC0 
-      12 SWITCH 
+      11 PUSHACC0
+      12 SWITCH
         int 0 -> 20
         tag 0 -> 20
         tag 1 -> 17
-      17 CONST0 
+      17 CONST0
       18 BRANCH 25
       20 GETGLOBAL Not_found
       22 MAKEBLOCK1 0
-      24 RAISE 
+      24 RAISE
       25 POP 1
-      27 ATOM0 
+      27 ATOM0
       28 SETGLOBAL T142-switch-A
-      30 STOP 
+      30 STOP
 **)
