@@ -1,3 +1,9 @@
+(* TEST
+   plugins="odoc_test.ml"
+   * ocamldoc
+    flags="-I ${ocamlsrcdir}/ocamldoc"
+*)
+
 module Foo = struct type t = int let x = 1 end;;
 module type MT = module type of Foo;;
 module Bar = struct type t = int let x = 2 end;;
