@@ -25,7 +25,7 @@ let () = print_endline (f M.eq) ;;
 [%%expect{|
 type (_, _) t = Any : ('a, 'b) t | Eq : ('a, 'a) t
 module M : sig type s = private [> `A ] val eq : (s, [ `A | `B ]) t end
-Line 16, characters 39-64:
+Lines 16-17, characters 39-16:
 16 | .......................................function
 17 |   | Any -> "Any"
 Warning 8: this pattern-matching is not exhaustive.
@@ -55,7 +55,7 @@ module N :
     type s = private < a : int; .. >
     val eq : (s, < a : int; b : bool >) t
   end
-Line 12, characters 49-74:
+Lines 12-13, characters 49-16:
 12 | .................................................function
 13 |   | Any -> "Any"
 Warning 8: this pattern-matching is not exhaustive.
