@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 #labels false;;
 type (_,_) eql = Refl : ('a, 'a) eql
 type s = x:int -> y:float -> unit
@@ -17,7 +21,9 @@ let f : [`L of (s, t) eql | `R of silly] -> 'a =
   function `R {silly} -> silly
 ;;
 [%%expect{|
-Line _, characters 2-30:
+Line 2, characters 2-30:
+2 |   function `R {silly} -> silly
+      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 `L Refl
