@@ -1,3 +1,11 @@
+(* TEST
+flags = " -w a "
+ocamlc_byte_exit_status = "2"
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+*** check-ocamlc.byte-output
+*)
+
 (* Sorry, we have to disable this as this requires accepting
    potentially badly formed programs (after expliciting) *)
 
