@@ -1,3 +1,10 @@
+(* TEST
+flags = " -w a "
+* setup-ocamlc.byte-build-env
+** ocamlc.byte
+*** check-ocamlc.byte-output
+*)
+
 (* OK *)
 module rec A : sig type 'a t = 'a array B.t * 'a list B.t end
              = struct type 'a t = 'a array B.t * 'a list B.t end
