@@ -24,9 +24,9 @@ let f = (fun x -> x + 1) [@inline never]
 
 let g x = (f [@inlined]) x
 
-let h = ref f
+let r = ref f
 
-let i x = (!h [@inlined]) x
+let i x = (!r [@inlined]) x
 
 let j x y = x + y
 
@@ -40,3 +40,13 @@ let b x y = (a [@inlined]) x y
 
 let c x = x + 1 [@@inline never]
 let d x = (c [@inlined]) x
+
+let g' x = (f [@inlined hint]) x
+
+let i' x = (!r [@inlined hint]) x
+
+let h' x = (j [@inlined hint]) x
+
+let b' x y = (a [@inlined hint]) x y
+
+let d' x = (c [@inlined hint]) x
