@@ -36,4 +36,4 @@ type link_mode =
   | MainDll
   | Partial
 
-val call_linker: link_mode -> string -> string list -> string -> bool
+val call_linker: link_mode -> string -> string list -> string -> int
